@@ -31,7 +31,5 @@ class Constants:
     PROP_SLICE_MODEL = "slice_model"
     PROP_BQM_MODEL = "bqm"
 
-    PROP_CAPACITIES = "capacities"
-    PROP_LABELS = "labels"
-    PROP_SLIVER_NAME = "name"
-    PROP_MANAGEMENT_IP = "management_ip"
+    PROP_SLIVER = "sliver"
+    PROP_SLIVER_TYPE = "sliver_type"
